
import java.util.*;

import java.math.BigInteger;
import java.security.KeyPairGenerator;
import java.security.SecureRandom;
import java.security.Security;

import org.bouncycastle.jce.ECNamedCurveTable;

import org.bouncycastle.jce.spec.ECParameterSpec;
import org.bouncycastle.math.ec.ECCurve;
import org.bouncycastle.math.ec.ECFieldElement;
import org.bouncycastle.math.ec.ECPoint;
import org.bouncycastle.math.ec.ECPoint.Fp;




public class Erik {
	
	ECParameterSpec ecSpec;
	ECCurve.Fp curve;
	ECPoint.Fp P; // the base point for the EC curve
	ECPoint.Fp Pinf; // the infinity point for the EC curve
	BigInteger n;//the order N of G, n is the smallest number such that nP = infinity point
	BigInteger q; // parameter of the finite field Fq
	byte[] curve_seed; //seed for generating curve parameters
	ECPoint.Fp public_key;
	private byte[] sk; // secret Key bytes
	private BigInteger secret_key; //secret Key
	SecureRandom rand; // Secure Pseudo Random number generator
	
	public Erik(){
		ecSpec = ECNamedCurveTable.getParameterSpec("secp112r1"); // Elliptic Curve prime192v1 size = 192 bits
		curve = (ECCurve.Fp)ecSpec.getCurve();
		P=(ECPoint.Fp)ecSpec.getG();
		q = curve.getQ();
		Pinf=(ECPoint.Fp)curve.getInfinity();
		curve_seed = ecSpec.getSeed();
		this.rand = new SecureRandom();
		this.rand.setSeed(this.curve_seed); // I initialize the Random numer generator with the same seed used for generating the curve
		
		n= ecSpec.getN();
		this.secret_key = this.createPrivateKey();
		this.public_key =(ECPoint.Fp) P.multiply(secret_key);
	}
	/**
	 * create the private key for this cipher
	 * @return BigIntger encapsulating the bytes of the secret key
	 */
	private BigInteger createPrivateKey(){
		int q_bit_length = this.q.bitLength();
		int priv_key_size = (int) (q_bit_length/8) - 1 ; /* The secret key belongs to Fq and is used in point multiplication
		 * to produce the public key, and we know that all elements
		 * in Fq are integers between 0 and q -1, and in particular,
		 * the scalars used for point multiplicaton should be between 0 and order_of(P) - 1 
		 * for simplicity we choose the secret key to be an order of byte
		 * smaller than the order n
		 **/
		sk = new byte[priv_key_size];
		rand.nextBytes(sk); // fill sk with random bytes ==> generate the secret key
		return new BigInteger(sk);
	}
	/**
	 * ECPoint.Fp 
	 * getPublicKEy()
	 * @return the public key for this cipher
	 */
	public ECPoint.Fp getPublicKEy(){
		return this.public_key;
	}
	
	/**
	 * ECParameterSpec 
	 * getECParamaterSpec()
	 * @return the parameter Spec for the elliptic curve associated with this cipher
	 */
	
	public ECParameterSpec getECParamaterSpec(){
		return ecSpec;
	}
	
	/**
	 * ECCurve.Fp 
	 * getCurve()
	 * @return the elliptic curve associated with this cipher
	 */
	public ECCurve.Fp getCurve(){
		return this.curve;
	}
	
	/**
	 * SecureRandom 
	 * getSecureRandom()
	 * @return the secure Random number generator associated with this cipher's elliptic Parameter Specification
	 */
	public SecureRandom getSecureRandom(){
		return this.rand;
	}
	
	/**
	 * ECPoint.Fp 
	 * getBasePoint()
	 * @return the base point of the elliptic curve associated with this Cipher
	 */
	public ECPoint.Fp getBasePoint(){
		return this.P;
	}
	
	/**
	 * ECPoint.Fp 
	 * getInfinityPoint()
	 * @return the infinity point of the elliptic curve associated with this Cipher
	 */
	public ECPoint.Fp getInfinityPoint(){
		return this.Pinf;
	}
	
	/**
	 * int getKeySize()
	 * @return Key size in bytes of the private key
	 */
	public int getKeySize(){
		return secret_key.bitLength();
	}
	
	/**
	 * Encrypts an elliptic curve point using the El Gamal encryption scheme
	 * @param X the point X is the mapping of plaintext x to a point on the curve
	 * @param r random number
	 * @param sk_size secret key size
	 * @return array of 2 ECPoints representing the encryption of input ECPoint
	 */
	public ECPoint.Fp[] encryptV2(ECPoint.Fp X, byte r[],int sk_size){ // the point X is the mapping of plaintext x to a point on the curve
		BigInteger R = new BigInteger(r);
		ECPoint.Fp u = (ECPoint.Fp)P.multiply(R); // rP
		ECPoint.Fp prod = (ECPoint.Fp)public_key.multiply(R); // R*Y, Y = pub key
		ECPoint.Fp v = (ECPoint.Fp)prod.add(X); // R*Y + X
		return new ECPoint.Fp[]{u,v};
	}
	
	/**
	 * ECPoint.Fp[] 
	 * encrypt(ECPoint.Fp X, long r)
	 * Encrypts an elliptic curve point using the El Gamal encryption scheme
	 * @param X the point X is the mapping of plaintext x to a point on the curve
	 * @param r random number
	 * @return array of 2 ECPoints representing the encryption of input ECPoint
	 */
	public ECPoint.Fp[] encrypt(ECPoint.Fp X, long r){ // the point X is the mapping of plaintext x to a point on the curve
		BigInteger R = BigInteger.valueOf(r);
		ECPoint.Fp u = (ECPoint.Fp)P.multiply(R); // rP
		ECPoint.Fp prod = (ECPoint.Fp)public_key.multiply(R); // R*Y, Y = pub key
		ECPoint.Fp v = (ECPoint.Fp)prod.add(X); // R*Y + X
		return new ECPoint.Fp[]{u,v};
	}
	
	/**
	 * ECPoint.Fp decrypt(ECPoint.Fp[] C)
	 * Decrypts a ciphetext C = (u,v) according to the scheme : D(C) = v - sk.u
	 * @param C ciphertext = (u,v)
	 * @return plaintext D(C) = v - sk.u
	 */
	public ECPoint.Fp decrypt(ECPoint.Fp[] C){ // C = (u,v)
		ECPoint.Fp Q = (ECPoint.Fp)C[1].subtract((ECPoint.Fp)C[0].multiply(secret_key)); // Q = v - sk.u
		return Q;
	}
	
	public static void printPoint(String name, ECPoint.Fp Pt){
		System.out.println(name +": ( "+ Pt.getX().toBigInteger() +","+ Pt.getY().toBigInteger() + ")");
	} 
	
	public static void main(String[] args) throws Exception {
		
		Security.addProvider(new org.bouncycastle.jce.provider.BouncyCastleProvider());

		/*Enumeration e = ECNamedCurveTable.getNames();

		while (e.hasMoreElements())
		    System.out.println(e.nextElement());
		    System.exit(0);*/



		Erik ecEG = new Erik();
		ECCurve.Fp curve = ecEG.getCurve();
		
		ECPoint.Fp P = ecEG.getBasePoint();
		ECPoint.Fp Pinf = ecEG.getInfinityPoint();
		SecureRandom rnd = ecEG.getSecureRandom();
		
		ECPoint.Fp Q;
		 		
		Q=(ECPoint.Fp)P.add(P);

		long old=Calendar.getInstance().getTimeInMillis();
		for (long i=1;i<=100000;i++) {
		Q=(ECPoint.Fp)P.add(Q);    
		}
		
		System.out.println("Elapsed: "+ (Calendar.getInstance().getTimeInMillis()-old));
		System.exit(0);


		System.out.println("The finite field for this curve is Fq with q= "+ecEG.q.toString());
		int field_size = curve.getFieldSize(); //equivalent to  return q.bitLength();
		System.out.println("curve field size = "+field_size);
		System.out.println("size = "+ecEG.q.bitLength());
		System.out.println("Order of base point P : "+ecEG.n.toString() + " bit length : "+ecEG.n.bitLength());
		System.out.println("private key size : "+ecEG.getKeySize() + " bytes");
		
		// Mapping : plaintext --> point on curve
		/* x | P(x)
		 * --|----
		 * 0 |Pinf
		 * --|----
		 * 1 |P
		 * 
		 * e.g: E(0) = (r.P, r.pub_key+Pinf)
		 */
		
		// C1 = E(0) = (u1,v1) tuple of Points with 0 ==> Pinf 
		/*byte[] r1 = new byte[priv_key_size];
		rnd.nextBytes(r1); // fill r1 with a random number
		*/
		
		long r1 = Math.abs(rnd.nextLong()); // generate a random number
		//long r1 = 2;
		ECPoint.Fp[] Enc_0 = ecEG.encrypt(Pinf,r1);
		
		long r2 = Math.abs(rnd.nextLong());
		//long r2 = 3;
		ECPoint.Fp[] Enc_1 = ecEG.encrypt(P,r2);
		
		System.out.println();
		
		// Now I compute E(P + Pinf) == E(1 + 0) ==  E(1) == E(P)
		ECPoint.Fp P_plus_Pinf = (ECPoint.Fp)P.add(Pinf);
		ECPoint.Fp P_plus_2Pinf = (ECPoint.Fp)P_plus_Pinf.add(Pinf);
		
		printPoint("   P    ", P);
		printPoint("P + Pinf", P_plus_Pinf);
		printPoint("P + 2.Pinf", P_plus_2Pinf);
		
		if(P.equals(P_plus_Pinf)){
			System.out.println("Yeah it's true that P + Pinf = P");
		}
		
		System.out.println(P_plus_2Pinf.equals(P)?"P + 2Pinf == Pinf":"P + 2Pinf != Pinf" );
		
		long r = r1 + r2 ;
		System.out.println("r == "+r);
		ECPoint.Fp[] result_r2 = ecEG.encrypt(P_plus_Pinf,r2);
		ECPoint.Fp[] result_r = ecEG.encrypt(P_plus_Pinf,r);
		System.out.println("Comparison : E(P + Pinf) and E(P)");
		
		printPoint("U_(Enc(P + Pinf) with r)  ",result_r[0]);
		
		ECPoint.Fp r1_x_P = (ECPoint.Fp)P.multiply(BigInteger.valueOf(r1));
		ECPoint.Fp r2_x_P = (ECPoint.Fp)P.multiply(BigInteger.valueOf(r2));
		
		ECPoint.Fp u1_plus_u2 = (ECPoint.Fp)r1_x_P.add(r2_x_P); // this should be equal to (r1+r2)*P
		printPoint("        r1.P + r2.P       ", u1_plus_u2);
		printPoint("           U1 + U2        ", (ECPoint.Fp)Enc_0[0].add(Enc_1[0]));
		printPoint("U_(Enc(P + Pinf) with r2) ",result_r2[0]);
		printPoint("           U_Enc_1        ",Enc_1[0]);
		
		printPoint("V_Enc(P + Pinf) with r2)  ",result_r2[1]);
		printPoint("           V_Enc_1        ",Enc_1[1]);
		
		printPoint("V_Enc(P + Pinf) with r)   ",result_r[1]);
		
		printPoint("     V_Enc(0) +V_Enc(1)   ", (ECPoint.Fp)Enc_0[1].add(Enc_1[1]));
		
		//Decryption tests
		
		System.out.println("Decryption :");
		
		//ECPoint.Fp Dec_1 = (ECPoint.Fp)Enc_1[1].subtract((ECPoint.Fp)Enc_1[0].multiply(secret_key));
		ECPoint.Fp Dec_0 = ecEG.decrypt(Enc_0);
		System.out.println(Dec_0.equals(Pinf)?"Dec(E(0)) == Pinf" : "Dec(E(0)) != Pinf");
		ECPoint.Fp Dec_1 = ecEG.decrypt(Enc_1);
		System.out.println(Dec_1.equals(P)?"Dec(E(1)) == P" : "Dec(E(1)) != P");
		
		//ECPoint.Fp Dec_P_plus_Pinf = (ECPoint.Fp)result_r[1].subtract((ECPoint.Fp)result_r[0].multiply(secret_key));
		ECPoint.Fp Dec_P_plus_Pinf = ecEG.decrypt(result_r);
		System.out.println(Dec_P_plus_Pinf.equals(P)?"Dec(E(0+1)) == P == Dec(E(1))" : "Dec(E(P+Pinf)) != P");
		
		//Tests
		System.out.println("Other tests on Point multiplication");
		ECPoint.Fp P1 = (ECPoint.Fp)P.multiply(BigInteger.valueOf(1)); // P1 = 1.P == P
		printPoint("    P   ",P1);
		printPoint("P1 = 1.P",P1);
		
		ECPoint.Fp P2 = (ECPoint.Fp)P.multiply(BigInteger.valueOf(2)); // P2 == 2P
		
		ECPoint.Fp P3 = (ECPoint.Fp)P1.add(P2); // P3 = P1 + P2 = P + 2P == 3P
		
		ECPoint.Fp P4 = (ECPoint.Fp)P.multiply(BigInteger.valueOf(3)); // P4 == 3P
		// P3 must be equal to P4
		printPoint("P3",P3);
		printPoint("P4",P4);
		System.out.println(P3.equals(P4)?"P3==P4" : "P3!=P4");
	}

}
