import java.io.DataOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FSDataOutputStream;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.NullWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.compress.CompressionCodec;
import org.apache.hadoop.io.compress.GzipCodec;
import org.apache.hadoop.mapreduce.OutputFormat;
import org.apache.hadoop.mapreduce.RecordWriter;
import org.apache.hadoop.mapreduce.TaskAttemptContext;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
import org.apache.hadoop.util.ReflectionUtils;

/** An {@link OutputFormat} that serialize ArrayList<BigInteger> objects to output file */
/** I use '|' (the bash pipe) as the key value separator, because I wanted a 1 byte character that is not likely to be in the key or in th value*/ 
public class SearchOutputFormat<K,V> extends FileOutputFormat<K, V> {
	  protected static class LineRecordWriter<K, V>
	    extends RecordWriter<K, V> {
		  
	    private static final String utf8 = "UTF-8";
	    
	    /* FOR EVENTUAL NEW LINE NEEDS
	    private static final byte[] newline;
	    static {
	      try {
	        newline = "\n".getBytes(utf8);
	      } catch (UnsupportedEncodingException uee) {
	        throw new IllegalArgumentException("can't find " + utf8 + " encoding");
	      }
	    }
	    */
		
	    protected DataOutputStream out;
	    private final byte[] keyValueSeparator;

	    public LineRecordWriter(DataOutputStream out, String keyValueSeparator) throws IOException {
	      this.out = out;
	      try {
	        this.keyValueSeparator = keyValueSeparator.getBytes(utf8);
	      } catch (UnsupportedEncodingException uee) {
	        throw new IllegalArgumentException("can't find " + utf8 + " encoding");
	      }
	    }

	    public LineRecordWriter(DataOutputStream out) throws IOException {
	      this(out, "|");
	    }

	    /**
	     * Write the object to the byte stream
	     * @param o the ArrayList of ECPoint.Fp[] to write
	     * @throws IOException if the write throws, we pass it on
	     */
	    private void writeObject(Object o) throws IOException {
	    	if (o instanceof Text ) { // this is for the Key
	    		Text to = (Text) o;
	    		to.write(out);
	    	} 
	    	if (o instanceof BytesArrayWritable) {  // this is for the value
	    		BytesArrayWritable bw = (BytesArrayWritable)o;
	    		bw.write(out); // Write the Array to this output Stream
	    	} 
	    }

	    public synchronized void write(K key, V value)
	      throws IOException {

	      boolean nullKey = key == null || key instanceof NullWritable;
	      boolean nullValue = value == null || value instanceof NullWritable;
	      if (nullKey && nullValue) {
	        return;
	      }
	      if (!nullKey) {
	        writeObject(key);
	      }
	      if (!(nullKey || nullValue)) {
	        out.write(keyValueSeparator);
	      }
	      if (!nullValue) {
	        writeObject(value);
	      }
	      
	      /*
	      out.write(newline);
	      */
	      
	    }

	    public synchronized 
	    void close(TaskAttemptContext context) throws IOException {
	      out.close();
	    }
	  }

	  public RecordWriter<K, V> 
	         getRecordWriter(TaskAttemptContext job
	                         ) throws IOException, InterruptedException {
	    Configuration conf = job.getConfiguration();
	    boolean isCompressed = getCompressOutput(job);
	    /*String keyValueSeparator= conf.get("mapred.textoutputformat.separator",
	                                       "\t"); //Change the path to the keyValue separator, hadoop.mapred is deprecated, we should use hadoop.mapreduce instead 
	    */                               
	    String keyValueSeparator = "|";                                   
	    CompressionCodec codec = null;
	    String extension = "";
	    if (isCompressed) {
	      Class<? extends CompressionCodec> codecClass = 
	        getOutputCompressorClass(job, GzipCodec.class);
	      codec = (CompressionCodec) ReflectionUtils.newInstance(codecClass, conf);
	      extension = codec.getDefaultExtension();
	    }
	    Path file = getDefaultWorkFile(job, extension);
	    FileSystem fs = file.getFileSystem(conf);
	    if (!isCompressed) {
	      FSDataOutputStream fileOut = fs.create(file, false);
	      return new LineRecordWriter<K, V>(fileOut, keyValueSeparator);
	    } else {
	      FSDataOutputStream fileOut = fs.create(file, false);
	      return new LineRecordWriter<K, V>(new DataOutputStream
	                                        (codec.createOutputStream(fileOut)),
	                                        keyValueSeparator);
	    }
	  }

}
