#!/bin/bash
PRISM_HOME=`pwd` #this is the root directory of the PRISM_Project, which should also be the directory containing this script
JAVA_CLASS=DecryptSearchOutput

#HDFS=hdfs://iwak.eurecom.fr:9000  #Name node of our cluster
HDFS=hdfs://localhost:9000      #local namenode [for testing purpose]

function usage {
  echo "Usage : $0 <word_to_find> <input_directory (on HDFS)> [<secret_key>]"
}

type -P hadoop &>/dev/null || { echo " ### This script requires hadoop to be installed and the hadoop executable to be in the PATH variable, please install hadoop if not already installed and include the bin folder of hadoop setup in the PATH. Aborting. ###" >&2; exit 1; }

if [ $# -lt 2 ]
then
    echo "### missing parameter(s) ###"
    usage #call the usage() function
    exit
fi

word=$1
input_file_path=$2

if [ $# -ge 3 ] #if there is a 3rd parameter it should be the secret_key key
then
  secret_key=$3
else
  secret_key="DEFAULT"
fi


input_file_dir=`echo $input_file_path | awk -F '/' '{print $(NF);}'` 
#the previous command takes the second to last element of the input_file_path 
# e.g from input path "/user/foo/output/06-02-2011-hello.txt_q5_t6_cnt0/" it will take : "06-02-2011-hello.txt_q5_t6_cnt0"

echo "input_file dir : "$input_file_dir
#We retrieve the day, the number of rounds and the exponent for the number of rows from the input file name
day=`echo $input_file_dir | cut -d "-" -f1,2,3`
num_rounds=`echo $input_file_dir | grep -o -e '_q[0-9][0-9]*' | grep -o -e '[0-9][0-9]*'`
t_exponent=`echo $input_file_dir | grep -o '_t[0-9][0-9]*' | grep -o -e '[0-9][0-9]*'`

output_file=PRISM_OUTPUT_$word'_'$day'_q'$num_rounds'_t'$t_exponent.txt # the name of the  output file of the decoding phase

local_input_fname=LOCAL_$input_file_dir #the encrypted input_file name that will be downloaded from the HDFS and written on the local disk

bash compile_sources.sh # run the compile_sources.sh script

#============FETCH THE FILE==============

echo "*** Downloading input file from HDFS, the file will be stored on the current directory then will be removed after decoding*** "

#if the file is already present, we delete it and we download a newer version
if [ -e $local_input_fname ]
then
  echo "Local file : $local_input_fname already exist, it will be deleted before downloading new version"
  rm $local_input_fname
fi

#hadoop dfsadmin -safemode leave > /dev/null # quit the safemode


#Now with the getmerge command we retrieve all the part-0000, part-00001, ......,part-0000x files produced by the reducers and we merge them into 1 single file that is downloaded localy
hadoop fs -getmerge $HDFS/$input_file_path $local_input_fname


status=$? #return code of previous command

if [ $status -ne 0 ]
then
  echo "HDFS Error occured while downloading file. Aborting."
  exit 1
fi


echo -e "\n*** Decoding of the input_file ***"
echo -e $word $local_input_fname $output_file $t_exponent $num_rounds $day $secret_key

java -classpath .:$PRISM_HOME/classes/:$PRISM_HOME/src/:$PRISM_HOME/libs/*:$PRISM_HOME/libs/gnu-crypto-2.0.1-bin/lib/* $JAVA_CLASS $word $local_input_fname $output_file $t_exponent $num_rounds $day $secret_key

status=$? #return code pf previous java command 

if [ $status -ne 0 ]
then
  echo "### Error while decoding file. Aborting ###"
  exit 1
fi

echo "Removing temporary local file :$local_input_fname"
rm $local_input_fname

if [ ! -d $PRISM_HOME/output ] #if the output directory does not already exist
  then
    echo "****creating directory $PRISM_HOME/output which will contain the result of the Search algorithm***"
    mkdir output
fi

mv $output_file $PRISM_HOME/output #I move the output_file in the output folder

echo "*** Done! Opening output_file ./output/$output_file to see output of PRISM Search algorithm ***"

echo -e "\n"

cat ./output/$output_file
